@@ -24,14 +24,17 @@
 #include <sys/time.h>
 #include "cc-timezone-map.h"
 #include "set-timezone.h"
-#include "dt-lockbutton.h"
+#include "libgnome-control-center/cc-lockbutton.h"
+#include "date-endian.h"
 
-#include <gsettings-desktop-schemas/gdesktop-enums.h>
+#include <gdesktop-enums.h>
 #include <string.h>
 #include <stdlib.h>
 #include <libintl.h>
 #include <polkit/polkit.h>
 
+/* FIXME: This should be "Etc/GMT" instead */
+#define DEFAULT_TZ "Europe/London"
 #define GETTEXT_PACKAGE_TIMEZONES GETTEXT_PACKAGE "-timezones"
 
 G_DEFINE_DYNAMIC_TYPE (CcDateTimePanel, cc_date_time_panel, CC_TYPE_PANEL)
@@ -337,18 +340,6 @@ change_date (CcDateTimePanel *self)
   queue_set_datetime (self);
 }
 
-static void
-location_changed_cb (CcTimezoneMap   *map,
-                     TzLocation      *location,
-                     CcDateTimePanel *self)
-{
-  g_debug ("location changed");
-
-  self->priv->current_location = location;
-
-  queue_set_timezone (self);
-}
-
 static void
 region_changed_cb (GtkComboBox     *box,
                    CcDateTimePanel *self)
@@ -392,8 +383,6 @@ update_timezone (CcDateTimePanel *self)
 {
   CcDateTimePanelPrivate *priv = self->priv;
   GtkWidget *widget;
-  time_t t;
-  struct tm *ltime;
   gchar **split;
   GtkTreeIter iter;
   GtkTreeModel *model;
@@ -401,9 +390,6 @@ update_timezone (CcDateTimePanel *self)
   /* tz.c updates the local timezone, which means the spin buttons can be
    * updated with the current time of the new location */
 
-  t = time (NULL);
-  ltime = localtime (&t);
-
   split = g_strsplit (priv->current_location->zone, "/", 2);
 
   /* remove underscores */
@@ -458,6 +444,33 @@ update_timezone (CcDateTimePanel *self)
   g_strfreev (split);
 }
 
+static void
+location_changed_cb (CcTimezoneMap   *map,
+                     TzLocation      *location,
+                     CcDateTimePanel *self)
+{
+  CcDateTimePanelPrivate *priv = self->priv;
+  GtkWidget *region_combo, *city_combo;
+
+  g_debug ("location changed to %s/%s", location->country, location->zone);
+
+  self->priv->current_location = location;
+
+  /* Update the combo boxes */
+  region_combo = W("region_combobox");
+  city_combo = W("city_combobox");
+
+  g_signal_handlers_block_by_func (region_combo, region_changed_cb, self);
+  g_signal_handlers_block_by_func (city_combo, city_changed_cb, self);
+
+  update_timezone (self);
+
+  g_signal_handlers_unblock_by_func (region_combo, region_changed_cb, self);
+  g_signal_handlers_unblock_by_func (city_combo, city_changed_cb, self);
+
+  queue_set_timezone (self);
+}
+
 static void
 get_timezone_cb (CcDateTimePanel *self,
                  const gchar     *timezone,
@@ -471,7 +484,11 @@ get_timezone_cb (CcDateTimePanel *self,
     }
   else
     {
-      cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), timezone);
+      if (!cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), timezone))
+        {
+          g_warning ("Timezone '%s' is unhandled, setting %s as default", timezone, DEFAULT_TZ);
+          cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), DEFAULT_TZ);
+        }
       self->priv->current_location = cc_timezone_map_get_location (CC_TIMEZONE_MAP (self->priv->map));
       update_timezone (self);
     }
@@ -738,13 +755,48 @@ on_permission_changed (GPermission *permission,
 {
   CcDateTimePanelPrivate *priv = CC_DATE_TIME_PANEL (data)->priv;
   gboolean allowed;
-  GtkWidget *vbox;
 
   allowed = g_permission_get_allowed (permission);
 
-  vbox = (GtkWidget*) gtk_builder_get_object (priv->builder, "vbox");
+  /* All the widgets but the lock button and the 24h setting */
+  gtk_widget_set_sensitive (W("map-vbox"), allowed);
+  gtk_widget_set_sensitive (W("hbox2"), allowed);
+  gtk_widget_set_sensitive (W("alignment2"), allowed);
+  gtk_widget_set_sensitive (W("table1"), allowed);
+}
 
-  gtk_widget_set_sensitive (vbox, allowed);
+static void
+reorder_date_widget (DateEndianess           endianess,
+		     CcDateTimePanelPrivate *priv)
+{
+  GtkWidget *month, *day, *year;
+  GtkBox *box;
+
+  if (endianess == DATE_ENDIANESS_MIDDLE)
+    return;
+
+  month = W ("month-combobox");
+  day = W ("day-spinbutton");
+  year = W("year-spinbutton");
+
+  box = GTK_BOX (W("table1"));
+
+  switch (endianess) {
+  case DATE_ENDIANESS_LITTLE:
+    gtk_box_reorder_child (box, month, 0);
+    gtk_box_reorder_child (box, day, 0);
+    gtk_box_reorder_child (box, year, -1);
+    break;
+  case DATE_ENDIANESS_BIG:
+    gtk_box_reorder_child (box, month, 0);
+    gtk_box_reorder_child (box, year, 0);
+    gtk_box_reorder_child (box, day, -1);
+    break;
+  case DATE_ENDIANESS_MIDDLE:
+    /* Let's please GCC */
+    g_assert_not_reached ();
+    break;
+  }
 }
 
 static void
@@ -765,6 +817,7 @@ cc_date_time_panel_init (CcDateTimePanel *self)
   int ret;
   GtkWidget *lockbutton;
   GPermission *permission;
+  DateEndianess endianess;
 
   priv = self->priv = DATE_TIME_PANEL_PRIVATE (self);
 
@@ -797,6 +850,8 @@ cc_date_time_panel_init (CcDateTimePanel *self)
 
   /* set up date editing widgets */
   priv->date = g_date_time_new_now_local ();
+  endianess = date_endian_get_default (FALSE);
+  reorder_date_widget (endianess, priv);
 
   gtk_combo_box_set_active (GTK_COMBO_BOX (W ("month-combobox")),
                             g_date_time_get_month (priv->date) - 1);
@@ -880,7 +935,7 @@ cc_date_time_panel_init (CcDateTimePanel *self)
     }
 
   /* DtLockButton takes ownership of the permission */
-  lockbutton = dt_lock_button_new (permission);
+  lockbutton = cc_lock_button_new (permission);
   gtk_widget_set_margin_top (lockbutton, 12);
   gtk_widget_show (lockbutton);
   gtk_box_pack_end ((GtkBox *) gtk_builder_get_object (priv->builder, "hbox"),
