@@ -26,13 +26,15 @@
 #include <glib/gstdio.h>
 #include <polkit/polkit.h>
 #include <dbus/dbus-glib.h>
-#include <gsettings-desktop-schemas/gdesktop-enums.h>
+#include <gdesktop-enums.h>
 
 #include <cups/cups.h>
 
 #include <math.h>
 
-#include "pp-lockbutton.h"
+#include "libgnome-control-center/cc-lockbutton.h"
+#include "pp-new-printer-dialog.h"
+#include "pp-utils.h"
 
 G_DEFINE_DYNAMIC_TYPE (CcPrintersPanel, cc_printers_panel, CC_TYPE_PANEL)
 
@@ -55,6 +57,7 @@ struct _CcPrintersPanelPrivate
   GtkWidget *lock_button;
 
   cups_dest_t *dests;
+  gchar **dest_model_names;
   int num_dests;
   int current_dest;
 
@@ -70,6 +73,8 @@ struct _CcPrintersPanelPrivate
 
   GPermission *permission;
 
+  PpNewPrinterDialog *pp_new_printer_dialog;
+
   gpointer dummy;
 };
 
@@ -112,7 +117,12 @@ cc_printers_panel_dispose (GObject *object)
   int                     i;
 
   if (priv->num_dests > 0)
-    cupsFreeDests (priv->num_dests, priv->dests);
+    {
+      for (i = 0; i < priv->num_dests; i++)
+        g_free (priv->dest_model_names[i]);
+      g_free (priv->dest_model_names);
+      cupsFreeDests (priv->num_dests, priv->dests);
+    }
   priv->dests = NULL;
   priv->num_dests = 0;
   priv->current_dest = -1;
@@ -166,6 +176,16 @@ cc_printers_panel_class_finalize (CcPrintersPanelClass *klass)
 {
 }
 
+enum
+{
+  NOTEBOOK_INFO_PAGE = 0,
+  NOTEBOOK_JOBS_PAGE,
+  NOTEBOOK_OPTIONS_PAGE,
+  NOTEBOOK_NO_PRINTERS_PAGE,
+  NOTEBOOK_NO_CUPS_PAGE,
+  NOTEBOOK_N_PAGES
+};
+
 enum
 {
   PRINTER_ID_COLUMN,
@@ -173,7 +193,6 @@ enum
   PRINTER_PAUSED_COLUMN,
   PRINTER_DEFAULT_ICON_COLUMN,
   PRINTER_ICON_COLUMN,
-  PRINTER_IS_SEPARATOR_COLUMN,
   PRINTER_N_COLUMNS
 };
 
@@ -188,15 +207,19 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   GtkWidget              *widget;
   gboolean                test_page_command_available = FALSE;
   gboolean                sensitive;
+  gboolean                is_local = TRUE;
   gchar                  *printer_make_and_model = NULL;
+  gchar                  *printer_model = NULL;
   gchar                  *reason = NULL;
   gchar                 **available_commands = NULL;
   gchar                  *printer_commands = NULL;
   gchar                 **printer_reasons = NULL;
   gchar                  *marker_types = NULL;
   gchar                  *printer_name = NULL;
+  gchar                  *printer_type = NULL;
   gchar                  *active_jobs = NULL;
   gchar                  *supply_type = NULL;
+  gchar                  *printer_uri = NULL;
   gchar                  *location = NULL;
   gchar                  *status = NULL;
   guint                   num_jobs;
@@ -281,6 +304,11 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       priv->current_dest < priv->num_dests &&
       priv->dests != NULL)
     {
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "notebook");
+      if (gtk_notebook_get_current_page (GTK_NOTEBOOK (widget)) >= NOTEBOOK_NO_PRINTERS_PAGE)
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), NOTEBOOK_INFO_PAGE);
+
       for (i = 0; i < priv->dests[id].num_options; i++)
         {
           if (g_strcmp0 (priv->dests[id].options[i].name, "printer-location") == 0)
@@ -294,7 +322,56 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-commands") == 0)
             printer_commands = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-make-and-model") == 0)
-            printer_make_and_model = g_strdup (priv->dests[priv->current_dest].options[i].value);
+            printer_make_and_model = priv->dests[priv->current_dest].options[i].value;
+          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-uri-supported") == 0)
+            printer_uri = priv->dests[priv->current_dest].options[i].value;
+          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-type") == 0)
+            printer_type = priv->dests[priv->current_dest].options[i].value;
+        }
+
+      if (priv->dest_model_names[priv->current_dest] == NULL)
+        priv->dest_model_names[priv->current_dest] =
+          get_ppd_attribute (priv->dests[priv->current_dest].name, "ModelName");
+
+      printer_model = g_strdup (priv->dest_model_names[priv->current_dest]);
+
+      if (printer_model == NULL && printer_make_and_model)
+        {
+          gchar *breakpoint = NULL, *tmp = NULL, *tmp2 = NULL;
+          gchar  backup;
+          size_t length = 0;
+          gchar *forbiden[] = {
+              "foomatic",
+              ",",
+              "hpijs",
+              "hpcups",
+              "(recommended)",
+              "postscript (recommended)",
+              NULL };
+
+          tmp = g_ascii_strdown (printer_make_and_model, -1);
+
+          for (i = 0; i < g_strv_length (forbiden); i++)
+            {
+              tmp2 = g_strrstr (tmp, forbiden[i]);
+              if (breakpoint == NULL || 
+                  (tmp2 != NULL && tmp2 < breakpoint))
+                breakpoint = tmp2;
+            }
+
+          if (breakpoint)
+            {
+              backup = *breakpoint;
+              *breakpoint = '\0';
+              length = strlen (tmp);
+              *breakpoint = backup;
+              g_free (tmp);
+
+              if (length > 0)
+                printer_model = g_strndup (printer_make_and_model, length);
+            }
+          else
+            printer_model = g_strdup (printer_make_and_model);
         }
 
       /* Find the first of the most severe reasons
@@ -373,7 +450,7 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       else
         gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
 
-        
+
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-status-label");
 
@@ -401,15 +478,53 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-model-label");
 
-      if (printer_make_and_model)
+      if (printer_model)
         {
-          gtk_label_set_text (GTK_LABEL (widget), printer_make_and_model);
-          g_free (printer_make_and_model);
+          gtk_label_set_text (GTK_LABEL (widget), printer_model);
+          g_free (printer_model);
         }
       else
         gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
 
 
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-ip-address-label");
+
+      if (printer_type)
+        {
+          cups_ptype_t type;
+          type = atoi (printer_type);
+          is_local = !(type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT));
+        }
+
+      if (is_local)
+        {
+          gtk_label_set_text (GTK_LABEL (widget), "localhost");
+        }
+      else
+        {
+          if (printer_uri)
+            {
+              char scheme[HTTP_MAX_URI],
+              userpass[HTTP_MAX_URI],
+              server[HTTP_MAX_URI],
+              resource[HTTP_MAX_URI];
+              int port;
+
+              httpSeparateURI(HTTP_URI_CODING_ALL, printer_uri, scheme, sizeof(scheme), userpass,
+                              sizeof(userpass), server, sizeof(server), &port, resource,
+                              sizeof(resource));
+
+              if (server[0] != '\0')
+                gtk_label_set_text (GTK_LABEL (widget), server);
+              else
+                gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
+            }
+          else
+            gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
+        }
+
+
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-disable-switch");
 
@@ -494,9 +609,27 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-jobs-button");
       gtk_widget_set_sensitive (widget, TRUE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-icon");
+      gtk_widget_set_sensitive (widget, TRUE);
     }
   else
     {
+      http_t *http = NULL;
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "notebook");
+
+      http = httpConnectEncrypt (cupsServer (), ippPort (), cupsEncryption ());
+      if (http)
+        {
+          httpClose (http);
+          gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), NOTEBOOK_NO_PRINTERS_PAGE);
+        }
+      else
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), NOTEBOOK_NO_CUPS_PAGE);
+
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-name-label");
       gtk_label_set_text (GTK_LABEL (widget), "");
@@ -517,6 +650,10 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
         gtk_builder_get_object (priv->builder, "printer-ip-address-label");
       gtk_label_set_text (GTK_LABEL (widget), "");
 
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-jobs-label");
+      gtk_label_set_text (GTK_LABEL (widget), "");
+
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "print-test-page-button");
       gtk_widget_set_sensitive (widget, FALSE);
@@ -528,6 +665,10 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-jobs-button");
       gtk_widget_set_sensitive (widget, FALSE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-icon");
+      gtk_widget_set_sensitive (widget, FALSE);
     }
 
   widget = (GtkWidget*)
@@ -543,26 +684,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   gtk_widget_set_sensitive (widget, FALSE);
 }
 
-typedef struct {
-  gint printer_id;
-  gint usage;
-} PrinterUsage;
-
-static gint
-printer_usage_cmp (gconstpointer a,
-                   gconstpointer b)
-{
-  PrinterUsage *x = (PrinterUsage*) a;
-  PrinterUsage *y = (PrinterUsage*) b;
-
-  if (x->usage < y->usage)
-    return 1;
-  else if (x->usage == y->usage)
-    return 0;
-  else
-    return -1;
-}
-
 static void
 actualize_printers_list (CcPrintersPanel *self)
 {
@@ -572,16 +693,12 @@ actualize_printers_list (CcPrintersPanel *self)
   GtkTreeView            *treeview;
   GtkTreeIter             iter;
   cups_job_t             *jobs = NULL;
-  gboolean                has_used_printer = FALSE;
-  gboolean                has_separator = FALSE;
   gboolean                paused = FALSE;
   gboolean                valid = FALSE;
   gchar                  *current_printer_instance = NULL;
   gchar                  *current_printer_name = NULL;
   gchar                  *printer_icon_name = NULL;
   gchar                  *default_icon_name = NULL;
-  GList                  *tmp_list = NULL;
-  GList                  *usages = NULL;
   int                     current_dest = -1;
   int                     i, j;
   int                     num_jobs = 0;
@@ -598,8 +715,14 @@ actualize_printers_list (CcPrintersPanel *self)
     }
 
   if (priv->num_dests > 0)
-    cupsFreeDests (priv->num_dests, priv->dests);
+    {
+      for (i = 0; i < priv->num_dests; i++)
+        g_free (priv->dest_model_names[i]);
+      g_free (priv->dest_model_names);
+      cupsFreeDests (priv->num_dests, priv->dests);
+    }
   priv->num_dests = cupsGetDests (&priv->dests);
+  priv->dest_model_names = g_new0 (gchar *, priv->num_dests);
   priv->current_dest = -1;
 
   treeview = (GtkTreeView*)
@@ -610,47 +733,28 @@ actualize_printers_list (CcPrintersPanel *self)
                               G_TYPE_STRING,
                               G_TYPE_BOOLEAN,
                               G_TYPE_STRING,
-                              G_TYPE_STRING,
-                              G_TYPE_BOOLEAN);
+                              G_TYPE_STRING);
+
+  if (priv->num_dests == 0)
+    {
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter,
+                          PRINTER_ID_COLUMN, 0,
+      /* Translators: There are no printers available (none is configured or CUPS is not running) */
+                          PRINTER_NAME_COLUMN, _("No printers available"),
+                          PRINTER_PAUSED_COLUMN, TRUE,
+                          PRINTER_DEFAULT_ICON_COLUMN, NULL,
+                          PRINTER_ICON_COLUMN, NULL,
+                          -1);
+      gtk_widget_set_sensitive (GTK_WIDGET (treeview), FALSE);
+    }
+  else
+    gtk_widget_set_sensitive (GTK_WIDGET (treeview), TRUE);
 
   for (i = 0; i < priv->num_dests; i++)
-    {
-      PrinterUsage *usage = g_new (PrinterUsage, 1);
-      num_jobs = cupsGetJobs (&jobs, priv->dests[i].name, 1, CUPS_WHICHJOBS_ALL);
-      usage->printer_id = i;
-      usage->usage = num_jobs;
-      usages = g_list_append (usages, usage);
-      cupsFreeJobs (num_jobs, jobs);
-    }
-  num_jobs = 0;
-  jobs = NULL;
-
-  usages = g_list_sort (usages, printer_usage_cmp);
-
-  for (tmp_list = usages; tmp_list; tmp_list = tmp_list->next)
     {
       gchar *instance;
 
-      i = ((PrinterUsage*) tmp_list->data)->printer_id;
-      if (((PrinterUsage*) tmp_list->data)->usage == 0)
-        {
-          if (!has_separator && has_used_printer)
-            {
-              has_separator = TRUE;
-              gtk_list_store_append (store, &iter);
-              gtk_list_store_set (store, &iter,
-                                  PRINTER_ID_COLUMN, -1,
-                                  PRINTER_NAME_COLUMN, NULL,
-                                  PRINTER_PAUSED_COLUMN, FALSE,
-                                  PRINTER_DEFAULT_ICON_COLUMN, FALSE,
-                                  PRINTER_ICON_COLUMN, NULL,
-                                  PRINTER_IS_SEPARATOR_COLUMN, TRUE,
-                                  -1);
-            }
-        }
-      else
-        has_used_printer = TRUE;
-
       gtk_list_store_append (store, &iter);
 
       if (priv->dests[i].instance)
@@ -696,14 +800,12 @@ actualize_printers_list (CcPrintersPanel *self)
                           PRINTER_PAUSED_COLUMN, paused,
                           PRINTER_DEFAULT_ICON_COLUMN, default_icon_name,
                           PRINTER_ICON_COLUMN, printer_icon_name,
-                          PRINTER_IS_SEPARATOR_COLUMN, FALSE,
                           -1);
 
       g_free (instance);
       g_free (printer_icon_name);
       g_free (default_icon_name);
     }
-  g_list_free_full (usages, g_free);
 
   gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
 
@@ -787,10 +889,25 @@ set_cell_sensitivity_func (GtkTreeViewColumn *tree_column,
                            GtkTreeIter       *iter,
                            gpointer           func_data)
 {
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) func_data;
   gboolean                paused = FALSE;
 
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
   gtk_tree_model_get (tree_model, iter, PRINTER_PAUSED_COLUMN, &paused, -1);
 
+  if (priv->num_dests == 0)
+    g_object_set (G_OBJECT (cell),
+                  "ellipsize", PANGO_ELLIPSIZE_NONE,
+                  "width-chars", -1,
+                  NULL);
+  else
+    g_object_set (G_OBJECT (cell),
+                  "ellipsize", PANGO_ELLIPSIZE_END,
+                  "width-chars", 18,
+                  NULL);
+
   if (paused)
     g_object_set (cell,
                   "sensitive", FALSE,
@@ -801,23 +918,6 @@ set_cell_sensitivity_func (GtkTreeViewColumn *tree_column,
                   NULL);
 }
 
-static gboolean
-printers_row_separator_func (GtkTreeModel *model,
-                             GtkTreeIter  *iter,
-                             gpointer      data)
-{
-  gboolean separator;
-
-  gtk_tree_model_get (model,
-                      iter,
-                      PRINTER_IS_SEPARATOR_COLUMN,
-                      &separator,
-                      -1);
-
-  return separator;
-}
-
-
 static void
 populate_printers_list (CcPrintersPanel *self)
 {
@@ -863,11 +963,6 @@ populate_printers_list (CcPrintersPanel *self)
                                                      "icon-name", PRINTER_DEFAULT_ICON_COLUMN, NULL);
   gtk_tree_view_column_set_expand (column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-
-  gtk_tree_view_set_row_separator_func (GTK_TREE_VIEW (treeview),
-                                        printers_row_separator_func,
-                                        NULL,
-                                        NULL);
 }
 
 enum
@@ -1084,73 +1179,6 @@ enum
   ALLOWED_USERS_N_COLUMNS
 };
 
-static int
-ccGetAllowedUsers (gchar ***allowed_users, char *printer_name)
-{
-  const char * const   attrs[1] = { "requesting-user-name-allowed" };
-  http_t              *http;
-  ipp_t               *request = NULL;
-  gchar              **users = NULL;
-  ipp_t               *response;
-  char                 uri[HTTP_MAX_URI + 1];
-  int                  num_allowed_users = 0;
-
-  http = httpConnectEncrypt (cupsServer (),
-                             ippPort (),
-                             cupsEncryption ());
-
-  if (http || !allowed_users)
-    {
-      request = ippNewRequest (IPP_GET_PRINTER_ATTRIBUTES);
-
-      g_snprintf (uri, sizeof (uri), "ipp://localhost/printers/%s", printer_name);
-      ippAddString (request,
-                    IPP_TAG_OPERATION,
-                    IPP_TAG_URI,
-                    "printer-uri",
-                    NULL,
-                    uri);
-      ippAddStrings (request,
-                     IPP_TAG_OPERATION,
-                     IPP_TAG_KEYWORD,
-                     "requested-attributes",
-                     1,
-                     NULL,
-                     attrs);
-
-      response = cupsDoRequest (http, request, "/");
-      if (response)
-        {
-          ipp_attribute_t *attr = NULL;
-          ipp_attribute_t *allowed = NULL;
-
-          for (attr = response->attrs; attr != NULL; attr = attr->next)
-            {
-              if (attr->group_tag == IPP_TAG_PRINTER &&
-                  attr->value_tag == IPP_TAG_NAME &&
-                  !g_strcmp0 (attr->name, "requesting-user-name-allowed"))
-                allowed = attr;
-            }
-
-          if (allowed && allowed->num_values > 0)
-            {
-              int i;
-
-              num_allowed_users = allowed->num_values;
-              users = g_new (gchar*, num_allowed_users);
-
-              for (i = 0; i < num_allowed_users; i ++)
-                users[i] = g_strdup (allowed->values[i].string.text);
-            }
-          ippDelete(response);
-        }
-       httpClose (http);
-     }
-
-  *allowed_users = users;
-  return num_allowed_users;
-}
-
 static void
 actualize_allowed_users_list (CcPrintersPanel *self)
 {
@@ -1246,33 +1274,6 @@ populate_allowed_users_list (CcPrintersPanel *self)
                     "changed", G_CALLBACK (allowed_users_selection_changed_cb), self);
 }
 
-static DBusGProxy *
-get_dbus_proxy ()
-{
-  DBusGConnection *system_bus;
-  DBusGProxy      *proxy;
-  GError          *error;
-
-  error = NULL;
-  system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-  if (system_bus == NULL)
-    {
-      /* Translators: Program cannot connect to DBus' system bus */
-      g_warning (_("Could not connect to system bus: %s"),
-                 error->message);
-      g_error_free (error);
-      return NULL;
-    }
-
-  error = NULL;
-
-  proxy = dbus_g_proxy_new_for_name (system_bus,
-                                     MECHANISM_BUS,
-                                     "/",
-                                     MECHANISM_BUS);
-  return proxy;
-}
-
 static void
 job_process_cb (GtkButton *button,
                 gpointer   user_data)
@@ -1294,7 +1295,10 @@ job_process_cb (GtkButton *button,
 
   if (id >= 0)
     {
-      proxy = get_dbus_proxy ();
+      proxy = get_dbus_proxy (MECHANISM_BUS,
+                              "/",
+                              MECHANISM_BUS,
+                              TRUE);
 
       if (!proxy)
         return;
@@ -1387,7 +1391,10 @@ printer_disable_cb (GObject    *gobject,
 
   if (name)
     {
-      proxy = get_dbus_proxy ();
+      proxy = get_dbus_proxy (MECHANISM_BUS,
+                              "/",
+                              MECHANISM_BUS,
+                              TRUE);
 
       if (!proxy)
         return;
@@ -1634,7 +1641,10 @@ allowed_user_remove_cb (GtkToolButton *button,
 
   if (name && printer_name)
     {
-      proxy = get_dbus_proxy ();
+      proxy = get_dbus_proxy (MECHANISM_BUS,
+                              "/",
+                              MECHANISM_BUS,
+                              TRUE);
 
       if (!proxy)
         return;
@@ -1704,7 +1714,10 @@ allowed_user_add_cb (GtkCellRendererText *renderer,
 
   if (new_text && new_text[0] != '\0' && printer_name)
     {
-      proxy = get_dbus_proxy ();
+      proxy = get_dbus_proxy (MECHANISM_BUS,
+                              "/",
+                              MECHANISM_BUS,
+                              TRUE);
 
       if (!proxy)
         return;
@@ -1805,7 +1818,10 @@ printer_set_default_cb (GtkToggleButton *button,
 
   if (name)
     {
-      proxy = get_dbus_proxy ();
+      proxy = get_dbus_proxy (MECHANISM_BUS,
+                              "/",
+                              MECHANISM_BUS,
+                              TRUE);
 
       if (!proxy)
         return;
@@ -1838,6 +1854,58 @@ printer_set_default_cb (GtkToggleButton *button,
   }
 }
 
+static void
+new_printer_dialog_response_cb (GtkDialog *dialog,
+                                gint       response_id,
+                                gpointer   user_data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  pp_new_printer_dialog_free (priv->pp_new_printer_dialog);
+  priv->pp_new_printer_dialog = NULL;
+
+  if (response_id == GTK_RESPONSE_OK)
+    actualize_printers_list (self);
+  else if (response_id == GTK_RESPONSE_REJECT)
+    {
+      GtkWidget *message_dialog;
+
+      message_dialog = gtk_message_dialog_new (NULL,
+                                               0,
+                                               GTK_MESSAGE_ERROR,
+                                               GTK_BUTTONS_CLOSE,
+      /* Translators: Addition of the new printer failed. */
+                                               _("Failed to add new printer."));
+      g_signal_connect (message_dialog,
+                        "response",
+                        G_CALLBACK (gtk_widget_destroy),
+                        NULL);
+      gtk_widget_show (message_dialog);
+    }
+}
+
+static void
+printer_add_cb (GtkToolButton *toolbutton,
+                gpointer       user_data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  GtkWidget              *widget;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  widget = (GtkWidget*)
+    gtk_builder_get_object (priv->builder, "main-vbox");
+
+  priv->pp_new_printer_dialog = pp_new_printer_dialog_new (
+    GTK_WINDOW (gtk_widget_get_toplevel (widget)),
+    new_printer_dialog_response_cb,
+    self);
+}
+
 static void
 printer_remove_cb (GtkToolButton *toolbutton,
                    gpointer       user_data)
@@ -1858,7 +1926,10 @@ printer_remove_cb (GtkToolButton *toolbutton,
 
   if (name)
     {
-      proxy = get_dbus_proxy ();
+      proxy = get_dbus_proxy (MECHANISM_BUS,
+                              "/",
+                              MECHANISM_BUS,
+                              TRUE);
 
       if (!proxy)
         return;
@@ -1882,67 +1953,6 @@ printer_remove_cb (GtkToolButton *toolbutton,
   }
 }
 
-static ipp_t *
-execute_maintenance_command (const char *printer_name,
-                             const char *command,
-                             const char *title)
-{
-  http_t *http;
-  GError *error = NULL;
-  ipp_t  *request = NULL;
-  ipp_t  *response = NULL;
-  char    uri[HTTP_MAX_URI + 1];
-  int     fd = -1;
-
-  http = httpConnectEncrypt (cupsServer (),
-                             ippPort (),
-                             cupsEncryption ());
-
-  if (http)
-    {
-      request = ippNewRequest (IPP_PRINT_JOB);
-
-      g_snprintf (uri,
-                  sizeof (uri),
-                  "ipp://localhost/printers/%s",
-                  printer_name);
-
-      ippAddString (request,
-                    IPP_TAG_OPERATION,
-                    IPP_TAG_URI,
-                    "printer-uri",
-                    NULL,
-                    uri);
-
-      ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name",
-                    NULL, title);
-
-      ippAddString (request, IPP_TAG_JOB, IPP_TAG_MIMETYPE, "document-format",
-                    NULL, "application/vnd.cups-command");
-
-      gchar *file_name = NULL;
-      fd = g_file_open_tmp ("ccXXXXXX", &file_name, &error);
-
-      if (fd != -1 && !error)
-        {
-          FILE *file;
-
-          file = fdopen (fd, "w");
-          fprintf (file, "#CUPS-COMMAND\n");
-          fprintf (file, "%s\n", command);
-          fclose (file);
-
-          response = cupsDoFileRequest (http, request, "/", file_name);
-          g_unlink (file_name);
-        }
-
-      g_free (file_name);
-      httpClose (http);
-    }
-
-  return response;
-}
-
 static void
 printer_maintenance_cb (GtkButton *button,
                         gpointer   user_data)
@@ -2004,6 +2014,9 @@ on_permission_changed (GPermission *permission,
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-add-button");
   gtk_widget_set_sensitive (widget, is_authorized);
 
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-add-button2");
+  gtk_widget_set_sensitive (widget, is_authorized);
+
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-remove-button");
   gtk_widget_set_sensitive (widget, is_authorized);
 
@@ -2020,14 +2033,6 @@ on_permission_changed (GPermission *permission,
   gtk_widget_set_sensitive (widget, is_authorized);
 }
 
-enum
-{
-  NOTEBOOK_INFO_PAGE = 0,
-  NOTEBOOK_JOBS_PAGE,
-  NOTEBOOK_OPTIONS_PAGE,
-  NOTEBOOK_N_PAGES
-};
-
 static void
 go_back_cb (GtkButton *button,
             gpointer   user_data)
@@ -2089,6 +2094,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   /* initialize main data structure */
   priv->builder = gtk_builder_new ();
   priv->dests = NULL;
+  priv->dest_model_names = NULL;
   priv->num_dests = 0;
   priv->current_dest = -1;
 
@@ -2100,6 +2106,8 @@ cc_printers_panel_init (CcPrintersPanel *self)
   priv->num_allowed_users = 0;
   priv->current_allowed_user = -1;
 
+  priv->pp_new_printer_dialog = NULL;
+
   gtk_builder_add_objects_from_file (priv->builder,
                                      DATADIR"/printers.ui",
                                      objects, &error);
@@ -2129,6 +2137,14 @@ cc_printers_panel_init (CcPrintersPanel *self)
     gtk_builder_get_object (priv->builder, "job-release-button");
   g_signal_connect (widget, "clicked", G_CALLBACK (job_process_cb), self);
 
+  widget = (GtkWidget*)
+    gtk_builder_get_object (priv->builder, "printer-add-button");
+  g_signal_connect (widget, "clicked", G_CALLBACK (printer_add_cb), self);
+
+  widget = (GtkWidget*)
+    gtk_builder_get_object (priv->builder, "printer-add-button2");
+  g_signal_connect (widget, "clicked", G_CALLBACK (printer_add_cb), self);
+
   widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "printer-remove-button");
   g_signal_connect (widget, "clicked", G_CALLBACK (printer_remove_cb), self);
@@ -2211,7 +2227,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
     "org.opensuse.cupspkhelper.mechanism.printeraddremove", NULL, NULL, NULL);
   if (priv->permission != NULL)
     {
-      widget = pp_lock_button_new (priv->permission);
+      widget = cc_lock_button_new (priv->permission);
       gtk_widget_set_margin_top (widget, 12);
       gtk_widget_show (widget);
       box = (GtkWidget*) gtk_builder_get_object (priv->builder, "main-vbox");
