@@ -32,10 +32,11 @@
 #include <canberra-gtk.h>
 
 #include "gvc-channel-bar.h"
+#include "gvc-mixer-control.h"
 
 #define SCALE_SIZE 128
-#define ADJUSTMENT_MAX_NORMAL 65536.0 /* PA_VOLUME_NORM */
-#define ADJUSTMENT_MAX_AMPLIFIED 98304.0 /* 1.5 * ADJUSTMENT_MAX_NORMAL */
+#define ADJUSTMENT_MAX_NORMAL gvc_mixer_control_get_vol_max_norm(NULL)
+#define ADJUSTMENT_MAX_AMPLIFIED gvc_mixer_control_get_vol_max_amplified(NULL)
 #define ADJUSTMENT_MAX (bar->priv->is_amplified ? ADJUSTMENT_MAX_AMPLIFIED : ADJUSTMENT_MAX_NORMAL)
 #define SCROLLSTEP (ADJUSTMENT_MAX / 100.0 * 5.0)
 
@@ -449,7 +450,7 @@ gvc_channel_bar_scroll (GvcChannelBar *bar, GdkScrollDirection direction)
                 /* Switch side scroll to vertical */
                 if (direction == GDK_SCROLL_RIGHT)
                         direction = GDK_SCROLL_UP;
-                else if (GDK_SCROLL_LEFT)
+                else if (direction == GDK_SCROLL_LEFT)
                         direction = GDK_SCROLL_DOWN;
         }
 
@@ -829,14 +830,14 @@ gvc_channel_bar_class_init (GvcChannelBarClass *klass)
                                          g_param_spec_string ("low-icon-name",
                                                               "Icon Name",
                                                               "Name of icon to display for this stream",
-                                                              "audio-volume-low",
+                                                              "audio-volume-low-symbolic",
                                                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_HIGH_ICON_NAME,
                                          g_param_spec_string ("high-icon-name",
                                                               "Icon Name",
                                                               "Name of icon to display for this stream",
-                                                              "audio-volume-high",
+                                                              "audio-volume-high-symbolic",
                                                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_IS_AMPLIFIED,
@@ -872,8 +873,8 @@ gvc_channel_bar_init (GvcChannelBar *bar)
         bar->priv = GVC_CHANNEL_BAR_GET_PRIVATE (bar);
 
         bar->priv->base_volume = ADJUSTMENT_MAX_NORMAL;
-        bar->priv->low_icon_name = g_strdup ("audio-volume-low");
-        bar->priv->high_icon_name = g_strdup ("audio-volume-high");
+        bar->priv->low_icon_name = g_strdup ("audio-volume-low-symbolic");
+        bar->priv->high_icon_name = g_strdup ("audio-volume-high-symbolic");
 
         bar->priv->orientation = GTK_ORIENTATION_VERTICAL;
         bar->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0,
@@ -906,10 +907,10 @@ gvc_channel_bar_init (GvcChannelBar *bar)
         bar->priv->mute_box = gtk_alignment_new (0.5, 0.5, 0, 0);
         gtk_container_add (GTK_CONTAINER (bar->priv->mute_box), bar->priv->mute_button);
 
-        bar->priv->low_image = gtk_image_new_from_icon_name ("audio-volume-low",
+        bar->priv->low_image = gtk_image_new_from_icon_name ("audio-volume-low-symbolic",
                                                              GTK_ICON_SIZE_BUTTON);
         gtk_widget_set_no_show_all (bar->priv->low_image, TRUE);
-        bar->priv->high_image = gtk_image_new_from_icon_name ("audio-volume-high",
+        bar->priv->high_image = gtk_image_new_from_icon_name ("audio-volume-high-symbolic",
                                                               GTK_ICON_SIZE_BUTTON);
         gtk_widget_set_no_show_all (bar->priv->high_image, TRUE);
 
