@@ -84,14 +84,11 @@ generate_draw (GtkWidget        *widget,
         if (!gtk_widget_is_sensitive (widget))
                 return;
 
-        gtk_paint_expander (gtk_widget_get_style (widget),
-                            cr,
-                            gtk_widget_get_state (widget),
-                            widget,
-                            NULL,
-                            gtk_widget_get_allocated_width (widget) - 12,
-                            gtk_widget_get_allocated_height (widget) - 12,
-                            GTK_EXPANDER_EXPANDED);
+        down_arrow (gtk_widget_get_style_context (widget),
+                    cr,
+                    gtk_widget_get_allocated_width (widget) - 12,
+                    gtk_widget_get_allocated_height (widget) - 12,
+                    12, 12);
 }
 
 static void
@@ -348,7 +345,7 @@ accept_password_dialog (GtkButton        *button,
                 cursor = gdk_cursor_new_for_display (display, GDK_WATCH);
                 gdk_window_set_cursor (gtk_widget_get_window (um->dialog), cursor);
                 gdk_display_flush (display);
-                gdk_cursor_unref (cursor);
+                g_object_unref (cursor);
         }
         else {
                 um_user_set_password (um->user, mode, password, hint);
@@ -620,11 +617,11 @@ um_password_dialog_new (void)
         error = NULL;
         filename = UIDIR "/password-dialog.ui";
         if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-                filename = "../data/password-dialog.ui";
+                filename = "data/password-dialog.ui";
         if (!gtk_builder_add_from_file (builder, filename, &error)) {
                 g_error ("%s", error->message);
                 g_error_free (error);
-                exit (1);
+                return NULL;
         }
 
         um = g_new0 (UmPasswordDialog, 1);
